/*******************************************************************/
/* Bibliotheque de manipulation d'image au format TrueColor 32bitd */
/* image.c                                                         */
/*                                                                 */
/* Ecrit par : Daniel Lacroix (all rights reserved)                */
/*                                                                 */
/*******************************************************************/

#include <image.h>

/************************************/
/* renvoi les dimensions de l'image */
void image_size(image *pimage, int32 *width, int32 *height)
{
  *width  = pimage->width;
  *height = pimage->height;
}
/************************************/

/***************************/
/* cree une nouvelle image */
image* image_new(int32 width, int32 height)
{ image *vimage;

  if((vimage=(image *)malloc(sizeof(image))) == NULL)
  {
    perror("malloc failed ");
    exit(1);
  }
  vimage->width = width; vimage->height = height;
  
  if((vimage->buf=(pix *)malloc(width*height*sizeof(pix))) == NULL)
  {
    perror("malloc failed ");
    exit(1);
  }
  return(vimage);
}
/***************************/

/********************/
/* libere une image */
void image_free(image *pimage)
{
  free(pimage->buf);
  free(pimage);
}
/********************/

/*************************************************/
/* copy l'image psrc dans pdest, les deux images */
/* doivent avoir la meme taille                  */
void image_copy(image *pdest, image *psrc)
{ 
  memcpy(pdest->buf, psrc->buf, (psrc->width*psrc->height)<<2);
}
/*************************************************/

/****************************************************/
/* (x,y) = position dans pdest ou est colle psrc    */
void image_paste(image *pdest, image *psrc, int32 x, int32 y)
{
  int32  x1,y1,x2,y2;
  uint32 *src_ptr,*dst_ptr;
  uint32 line_size;

  src_ptr = (uint32 *)((void *)psrc->buf + (((x>=0)?0:-x)<<2) + ((((y>=0)?0:-y)*psrc->width)<<2));
  dst_ptr = (uint32 *)((void *)pdest->buf + (((x>=0)?x:0)<<2) + ((((y>=0)?y:0)*pdest->width)<<2));
  line_size = ((pdest->width-x<psrc->width)?pdest->width-x:psrc->width) - ((x>=0)?0:-x);
  
  for(y1=(y>=0)?0:-y; y1<((pdest->height-y<psrc->height)?pdest->height-y:psrc->height); y1++)
  {
    for(x1=(x>=0)?0:-x; x1<((pdest->width-x<psrc->width)?pdest->width-x:psrc->width); x1++)
    {
      *dst_ptr = *src_ptr;
      src_ptr++; dst_ptr++;
    }
    src_ptr += psrc->width  - line_size;
    dst_ptr += pdest->width - line_size;
  }
}
/****************************************************/

/****************************************************/
/* (x,y) = position dans pdest ou est incruste psrc */
/* et col la couleur d'incruste de psrc             */
void image_incruste_ext(image *pdest, image *psrc, int32 x, int32 y, 
  uint8 r, uint8 g, uint8 b, uint8 p)
{ pix col;
  col.r = r; col.g = g; col.b = b; col.p = p;
  image_incruste(pdest,psrc,x,y,col);
}

void image_incruste(image *pdest, image *psrc, int32 x, int32 y, pix col)
{
  int32  x1,y1,x2,y2;
  uint32 *src_ptr,*dst_ptr;
  uint32 line_size;

  src_ptr = (uint32 *)((void *)psrc->buf + (((x>=0)?0:-x)<<2) + ((((y>=0)?0:-y)*psrc->width)<<2));
  dst_ptr = (uint32 *)((void *)pdest->buf + (((x>=0)?x:0)<<2) + ((((y>=0)?y:0)*pdest->width)<<2));
  line_size = ((pdest->width-x<psrc->width)?pdest->width-x:psrc->width) - ((x>=0)?0:-x);
  
/*  printf("pix d'incrute : (%u,%u,%u,%u)\n",col.r,col.g,col.b,col.p);*/
  for(y1=(y>=0)?0:-y; y1<((pdest->height-y<psrc->height)?pdest->height-y:psrc->height); y1++)
  {
    for(x1=(x>=0)?0:-x; x1<((pdest->width-x<psrc->width)?pdest->width-x:psrc->width); x1++)
    {
      if(*src_ptr != *((uint32 *)&col)) *dst_ptr = *src_ptr;
      src_ptr++; dst_ptr++;
    }
    src_ptr += psrc->width  - line_size;
    dst_ptr += pdest->width - line_size;
  }
}
/****************************************************/

/****************************************************/
/* (x,y) = position dans pdest ou est incruste psrc */
/* et col la couleur d'incruste de psrc. rapport l' */
/* intensite de psrc sur 256 par rapport a pdest    */
void image_transparent_ext(image *pdest, image *psrc, int32 x, int32 y, 
  uint8 r, uint8 g, uint8 b, uint8 p, uint16 rapport)
{ pix col;
  col.r = r; col.g = g; col.b = b; col.p = p;
  image_transparent(pdest,psrc,x,y,col,rapport);
}

void image_transparent(image *pdest, image *psrc, int32 x, int32 y, pix col,
  uint16 rapport)
{
  int32  x1,y1,x2,y2;
  uint32 *src_ptr,*dst_ptr;
  uint32 line_size;

  src_ptr = (uint32 *)((void *)psrc->buf + (((x>=0)?0:-x)<<2) + ((((y>=0)?0:-y)*psrc->width)<<2));
  dst_ptr = (uint32 *)((void *)pdest->buf + (((x>=0)?x:0)<<2) + ((((y>=0)?y:0)*pdest->width)<<2));
  line_size = ((pdest->width-x<psrc->width)?pdest->width-x:psrc->width) - ((x>=0)?0:-x);
  
  for(y1=(y>=0)?0:-y; y1<((pdest->height-y<psrc->height)?pdest->height-y:psrc->height); y1++)
  {
    for(x1=(x>=0)?0:-x; x1<((pdest->width-x<psrc->width)?pdest->width-x:psrc->width); x1++)
    {
      if(*src_ptr != *((uint32 *)&col)) 
      {
        (*((pix *)dst_ptr)).r = (((*((pix *)dst_ptr)).r * rapport) + 
	                       ((*((pix *)src_ptr)).r * (256 - rapport)))>>8;
        (*((pix *)dst_ptr)).g = (((*((pix *)dst_ptr)).g * rapport) + 
	                       ((*((pix *)src_ptr)).g * (256 - rapport)))>>8;
        (*((pix *)dst_ptr)).b = (((*((pix *)dst_ptr)).b * rapport) + 
	                       ((*((pix *)src_ptr)).b * (256 - rapport)))>>8;
      }
      src_ptr++; dst_ptr++;
    }
    src_ptr += psrc->width  - line_size;
    dst_ptr += pdest->width - line_size;
  }
}
/****************************************************/

/*****************************************************/
/* converti l'image psrc en noir et blanc dans pdest */
void image_gris(image *pdest, image *psrc, int32 x, int32 y)
{ uint32 luminance;
  int32  x1,y1,x2,y2;
  uint32 *src_ptr,*dst_ptr;
  uint32 line_size;

  src_ptr = (uint32 *)((void *)psrc->buf + (((x>=0)?0:-x)<<2) + ((((y>=0)?0:-y)*psrc->width)<<2));
  dst_ptr = (uint32 *)((void *)pdest->buf + (((x>=0)?x:0)<<2) + ((((y>=0)?y:0)*pdest->width)<<2));
  line_size = ((pdest->width-x<psrc->width)?pdest->width-x:psrc->width) - ((x>=0)?0:-x);
  
  for(y1=(y>=0)?0:-y; y1<((pdest->height-y<psrc->height)?pdest->height-y:psrc->height); y1++)
  {
    for(x1=(x>=0)?0:-x; x1<((pdest->width-x<psrc->width)?pdest->width-x:psrc->width); x1++)
    {
      luminance = (( 76*(uint16)(*((pix *)src_ptr)).r)+
 	           (150*(uint16)(*((pix *)src_ptr)).g)+
	           ( 30*(uint16)(*((pix *)src_ptr)).b))>>8;
		  
      (*((pix *)dst_ptr)).r = (uint8)luminance;
      (*((pix *)dst_ptr)).g = (uint8)luminance;
      (*((pix *)dst_ptr)).b = (uint8)luminance;

      src_ptr++; dst_ptr++;
    }
    src_ptr += psrc->width  - line_size;
    dst_ptr += pdest->width - line_size;
  }
}
/*****************************************************/

/****************************************************/
/* (x,y) = position dans pdest ou est colle psrc    */
void image_paste_with_alpha(image *pdest, image *psrc, int32 x, int32 y)
{
  int32  x1,y1,x2,y2, x_end, y_end;
  uint32 *src_ptr,*dst_ptr;
  uint32 line_size;
  uint32 a, inv_a, tmp_r, tmp_g, tmp_b;

  src_ptr = (uint32 *)((void *)psrc->buf + (((x>=0)?0:-x)<<2) + ((((y>=0)?0:-y)*psrc->width)<<2));
  dst_ptr = (uint32 *)((void *)pdest->buf + (((x>=0)?x:0)<<2) + ((((y>=0)?y:0)*pdest->width)<<2));
  line_size = ((pdest->width-x<psrc->width)?pdest->width-x:psrc->width) - ((x>=0)?0:-x);
  
  y_end = ((pdest->height-y<psrc->height)?pdest->height-y:psrc->height);
  x_end = ((pdest->width-x<psrc->width)?pdest->width-x:psrc->width);
  for(y1=(y>=0)?0:-y; y1 < y_end; y1++)
  {
    for(x1=(x>=0)?0:-x; x1 < x_end; x1++)
    {
      a = ((pix *)src_ptr)->p;
      if(a == 255) {
        ((pix *)dst_ptr)->r = ((pix *)src_ptr)->r;
        ((pix *)dst_ptr)->g = ((pix *)src_ptr)->g;
        ((pix *)dst_ptr)->b = ((pix *)src_ptr)->b;
      } else if(a == 0) {
        ((pix *)dst_ptr)->r = ((pix *)dst_ptr)->r;
        ((pix *)dst_ptr)->g = ((pix *)dst_ptr)->g;
        ((pix *)dst_ptr)->b = ((pix *)dst_ptr)->b;
      } else {
        inv_a = 255 - a;
        ((pix *)dst_ptr)->r = ((((pix *)dst_ptr)->r * inv_a) +
                           (((pix *)src_ptr)->r * a)) /*/ 255*/ >> 8;
        ((pix *)dst_ptr)->g = ((((pix *)dst_ptr)->g * inv_a) +
                           (((pix *)src_ptr)->g * a)) /*/ 255*/ >> 8;
        ((pix *)dst_ptr)->b = ((((pix *)dst_ptr)->b * inv_a) +
                           (((pix *)src_ptr)->b * a)) /*/ 255*/ >> 8;
      }
      src_ptr++; dst_ptr++;
    }
    src_ptr += psrc->width  - line_size;
    dst_ptr += pdest->width - line_size;
  }
}
/****************************************************/

/***************************************************************/
/* (x,y) = position dans pdest ou psrc est utilise comme alpha */
void image_set_alpha_with_grey(image *pdest, image *psrc, int32 x, int32 y)
{
  int32  x1,y1,x2,y2;
  uint32 *src_ptr,*dst_ptr;
  uint32 line_size;

  src_ptr = (uint32 *)((void *)psrc->buf + (((x>=0)?0:-x)<<2) + ((((y>=0)?0:-y)*psrc->width)<<2));
  dst_ptr = (uint32 *)((void *)pdest->buf + (((x>=0)?x:0)<<2) + ((((y>=0)?y:0)*pdest->width)<<2));
  line_size = ((pdest->width-x<psrc->width)?pdest->width-x:psrc->width) - ((x>=0)?0:-x);
  
  for(y1=(y>=0)?0:-y; y1<((pdest->height-y<psrc->height)?pdest->height-y:psrc->height); y1++)
  {
    for(x1=(x>=0)?0:-x; x1<((pdest->width-x<psrc->width)?pdest->width-x:psrc->width); x1++)
    {
      ((pix *)dst_ptr)->p = ((pix *)src_ptr)->r;
      src_ptr++; dst_ptr++;
    }
    src_ptr += psrc->width  - line_size;
    dst_ptr += pdest->width - line_size;
  }
}
/***************************************************************/

