/****************************************************/
/* Quelques primitives de dessin                    */
/* image_draw_func.c                                */
/*                                                  */
/* Ecrit par : Daniel Lacroix (all rights reserved) */
/*                                                  */
/****************************************************/

#include <image_draw_func.h>
#include <color.h>

/*********************************************************/
/* dessine un point aux coordonnees (x,y) de couleur col */
void put_pix(image *pimage, int32 x, int32 y, pix col)
{
  if((x >= 0) && (y >= 0) && (x < pimage->width) && (y < pimage->height))
    pimage->buf[(y * pimage->width) + x] = col;
}
/*********************************************************/

/*****************************************/
/* renvoi le point aux coordonnees (x,y) */
pix get_pix(image *pimage, int32 x, int32 y)
{
  if((x >= 0) && (y >= 0) && (x < pimage->width) && (y < pimage->height))
    return(pimage->buf[(y * pimage->width) + x]);
	else
	  return(BLACK);
}
/*****************************************/

/*************************************************************/
/* dessine une ligne entre (x1,y1) et (x2,y2) de couleur col */
void line(image *pimage, int32 x1, int32 y1, int32 x2, int32 y2, pix col)
{ int32 x,y,d,deltax,deltay;
  uint32 *sbuf, *ebuf;
  int32 ligne,yval,xval;

  deltay = (y2 > y1)? y2 - y1 : y1 - y2;
  deltax = (x2 > x1)? x2 - x1 : x1 - x2;

  /* si deltax > deltay, on avance en x */
  if(deltax > deltay)
  {
    /* tri pour que x1 <= x2 */
    if(x2 < x1)
    {
      x = x1; x1 = x2; x2 = x;
      y = y1; y1 = y2; y2 = y;
    }

    ligne  = (y2 > y1)? pimage->width : -pimage->width;
    yval   = (y2 > y1)? 1 : -1;

    sbuf = (uint32 *)(&pimage->buf[y1*pimage->width+x1]);
    ebuf = (uint32 *)(&pimage->buf[y2*pimage->width+x2]);

    for(x=x1, y=y1, d=(deltay<<1)-deltax; x <= x2 - (deltax >> 1); x++)
    {
      /* Dessine un pixel si dans l'image */
      if((x >= 0) && (x < pimage->width) && (y >= 0) && (y < pimage->height))
        *sbuf = *((uint32 *)&col);
      sbuf++;
      /* Dessine le pixel symetrique par rapport au milieu si il est dans l'image */  
      if((x2-(x-x1) >= 0) && (x2-(x-x1) < pimage->width) && (y2-(y-y1) >= 0) && (y2-(y-y1) < pimage->height))
        *ebuf = *((uint32 *)&col);
      ebuf--;
      
      if(d < 0) {
        d += deltay << 1;
      } else {
        d += (deltay - deltax) << 1;
        y += yval;
        sbuf += ligne;
        ebuf -= ligne;
      }
    }

  /* si deltay >= deltax, on avance de y */
  } else {

    /* tri pour que y1 <= y2 */
    if(y2 < y1)
    {
      x = x1; x1 = x2; x2 = x;
      y = y1; y1 = y2; y2 = y;
    }

    xval = (x2 > x1)? 1 : -1;

    sbuf = (uint32 *)(&pimage->buf[y1*pimage->width+x1]);
    ebuf = (uint32 *)(&pimage->buf[y2*pimage->width+x2]);

    for(x=x1, y=y1, d=(deltax<<1)-deltay; y <= y2 - (deltay >> 1); y++)
    {
      /* Dessine le pixel si il est dans l'image */
      if((x >= 0) && (x < pimage->width) && (y >= 0) && (y < pimage->height))
        *sbuf = *((uint32 *)&col);
      sbuf += pimage->width;
      /* Dessine le pixel symetrique par rapport au milieu si il est dans l'image */
      if((x2-(x-x1) >= 0) && (x2-(x-x1) < pimage->width) && (y2-(y-y1) >= 0) && (y2-(y-y1) < pimage->height))
        *ebuf = *((uint32 *)&col);
      ebuf -= pimage->width;
      
      if(d < 0) {
        d += deltax << 1;
      } else {
        d += (deltax - deltay) << 1;
        x += xval;
        sbuf += xval;
        ebuf -= xval;
      }
    }
  }

}
/*************************************************************/

/*********************************************************************/
/* dessine un rectangle vide entre (x1,y1) et (x2,y2) de couleur col */
void rectangle(image *pimage, int32 x1, int32 y1, int32 x2, int32 y2,pix col)
{
  hline(pimage, x1, x2, y1, col);
  hline(pimage, x1, x2, y2, col);
  vline(pimage, x1, y1, y2, col);
  vline(pimage, x2, y1, y2, col);
}
/*********************************************************************/

/***********************************************************************/
/* dessine une ligne horizontale entre (x1,y) et (x2,y) de couleur col */
void hline(image *pimage, int32 x1, int32 x2, int32 y, pix col)
{ int32 x;
  uint32 *buf;

  if(x1>x2){x = x1; x1 = x2; x2 = x;}

  /* pour le clipping avec l'image */
  if((x2<0) || (y<0) || (x1>=pimage->width) || (y>=pimage->height)) return;
  if(x1<0) x1 = 0;
  if(x2>=pimage->width)  x2 = pimage->width-1;
  
  buf = (uint32 *)(&pimage->buf[y*pimage->width+x1]);

  for(x=x1;x<=x2;x++)
    *buf++ = *((uint32 *)&col);
}
/***********************************************************************/

/*********************************************************************/
/* dessine une ligne verticale entre (x,y1) et (x,y2) de couleur col */
void vline(image *pimage, int32 x, int32 y1, int32 y2, pix col)
{ int32 y;
  uint32 *buf;

  if(y1>y2){y = y1; y1 = y2; y2 = y;}

  /* pour le clipping avec l'image */
  if((x<0) || (y2<0) || (x>=pimage->width) || (y1>=pimage->height)) return;
  if(y1<0) y1 = 0;
  if(y2>=pimage->height)  y2 = pimage->height-1;
  
  buf = (uint32 *)(&pimage->buf[y1*pimage->width+x]);

  for(y=y1;y<=y2;y++)
  {
    *buf = *((uint32 *)&col);
    buf += pimage->width;
  }
}
/*********************************************************************/

/*******************************/
/* dessine un rectangle rempli */
void rectangle_fill(image *pimage, int32 x1, int32 y1, int32 x2, int32 y2,pix col)
{ int32 x,y,ligne;
  uint32 *buf;
  
  if(x1>x2){x = x1; x1 = x2; x2 = x;}
  if(y1>y2){y = y1; y1 = y2; y2 = y;}
  
  if((x2<0) || (y2<0) || (x1>=pimage->width) || (y1>=pimage->height)) return;
  
  if(x1<0) x1 = 0; if(y1<0) y1 = 0;
  if(x2>=pimage->width)  x2 = pimage->width-1;
  if(y2>=pimage->height) y2 = pimage->height-1;

  buf = (uint32 *)(&pimage->buf[y1*pimage->width+x1]);

  for(y=y1;y<=y2;y++)
  {
    for(x=x1;x<=x2;x++)
      *buf++ = *((uint32 *)&col);
    buf += pimage->width - (x2-x1+1);
  }
}
/*******************************/

/*****************************/
/* repaint le fond avec col  */
void clear_img(image *pimage,pix col)
{ uint32 compt;
  
  /* place la couleur col dans tout le buffer image */
  for(compt=0;compt<pimage->width*pimage->height;compt++)
    pimage->buf[compt] = col;
}
/*****************************/

/********************************************************************************/
/* Dessine une ellipse de centre (cx,cy) et de rayon horizontal rx, vertical ry */
void ellipse(image *pimage, int32 cx, int32 cy, uint32 rx, uint32 ry, pix col)
{
  /* intermediate terms to speed up loop */
  int32 t1 = rx*rx, t2 = t1<<1, t3 = t2<<1;
  int32 t4 = ry*ry, t5 = t4<<1, t6 = t5<<1;
  int32 t7 = rx*t5, t8 = t7<<1, t9 = 0L;
  int32 d1 = t2 - t7 + (t4>>1);    /* error terms */
  int32 d2 = (t1>>1) - t8 + t5;

  int32 x = rx, y = 0;     /* ellipse points */

  while (d2 < 0)                  /* til slope = -1 */
  {
    /* draw 4 points using symmetry */
    put_pix(pimage, cx + x, cy + y, col);
    put_pix(pimage, cx + x, cy - y, col);
    put_pix(pimage, cx - x, cy + y, col);
    put_pix(pimage, cx - x, cy - y, col);

    y++;            /* always move up here */
    t9 += t3;       
    if (d1 < 0)     /* move straight up */
    {
      d1 += t9 + t2;
      d2 += t9;
    } else {           /* move up and left */
      x--;
      t8 -= t6;
      d1 += t9 + t2 - t8;
      d2 += t9 + t5 - t8;
    }
  }

  do                              /* rest of top right quadrant */
  {
    /* draw 4 points using symmetry */
    put_pix(pimage, cx + x, cy + y, col);
    put_pix(pimage, cx + x, cy - y, col);
    put_pix(pimage, cx - x, cy + y, col);
    put_pix(pimage, cx - x, cy - y, col);

    x--;            /* always move left here */
    t8 -= t6;       
    if(d2 < 0)     /* move up and left */
    {
      y++;
      t9 += t3;
      d2 += t9 + t5 - t8;
    } else            /* move straight left */
      d2 += t5 - t8;
  } while(x >= 0);
}
/********************************************************************************/

/******************************************************/
/* Dessine une ellipse rempli de centre (cx,cy) et de */
/* rayon horizontal rx, vertical ry                   */
void ellipse_fill(image *pimage, int32 cx, int32 cy, uint32 rx, uint32 ry, pix col)
{

  int32 t1 = rx*rx, t2 = t1<<1, t3 = t2<<1;
  int32 t4 = ry*ry, t5 = t4<<1, t6 = t5<<1;
  int32 t7 = rx*t5, t8 = t7<<1, t9 = 0;
  int32 d1 = t2 - t7 + (t4>>1);    /* error terms */
  int32 d2 = (t1>>1) - t8 + t5;
  int32 x = rx, y = 0;     /* ellipse points */
  int32 t;                 /* used in fill operation */

  while (d2 < 0)                  /* til slope = -1 */
  {
    /* fill in leftward to inner ellipse */
    hline(pimage, cx+x, cx-x, cy-y, col);
    hline(pimage, cx+x, cx-x, cy+y, col);

    y++;            /* always move up here */
    t9 += t3;       
    if (d1 < 0)     /* move straight up */
    {
      d1 += t9 + t2;
      d2 += t9;
    } else {            /* move up and left */
      x--;
      t8 -= t6;
      d1 += t9 + t2 - t8;
      d2 += t9 + t5 - t8;
    }
  }

  do                              /* rest of top right quadrant */
  {
    /* fill in downward to inner ellipse */
    hline(pimage, cx+x, cx-x, cy-y, col);
    hline(pimage, cx+x, cx-x, cy+y, col);

    x--;            /* always move left here */
    t8 -= t6;       
    if (d2 < 0)     /* move up and left */
    {
      y++;
      t9 += t3;
      d2 += t9 + t5 - t8;
    } else {           /* move straight left */
      d2 += t5 - t8;
    }
  } while(x >= 0);
}
/******************************************************/

/*****************************************************************/
/* dessine une ligne horizontale entre (x1,y) de couleur col1    */
/* et (x2,y) de couleur col2 avec un degrade lineaire de couleur */
void hline_shade(image *pimage, int32 x1, int32 x2, int32 y, pix col1, pix col2)
{ int32 x;
  pix col;
  uint32 *buf;

  if(x1>x2){x = x1; x1 = x2; x2 = x; col = col1; col1 = col2; col2 = col;}

  /* pour le clipping avec l'image */
  if((x2<0) || (y<0) || (x1>=pimage->width) || (y>=pimage->height)) return;
  
  buf = (uint32 *)(&pimage->buf[y*pimage->width+MAX(0,x1)]);
  
  for(x = MAX(0,x1); x < MIN(pimage->width,x2+1); x++)
  {
    col.r = (col1.r * (x2-x) / (x2-x1)) + (col2.r * (x-x1) / (x2-x1));
    col.g = (col1.g * (x2-x) / (x2-x1)) + (col2.g * (x-x1) / (x2-x1));
    col.b = (col1.b * (x2-x) / (x2-x1)) + (col2.b * (x-x1) / (x2-x1));
    col.p = 255;
    *buf++ = *((uint32 *)&col);
  }
}
/*****************************************************************/

/*****************************************************************/
/* dessine une ligne verticale entre (x,y1) de couleur col1      */
/* et (x,y2) de couleur col2 avec un degrade lineaire de couleur */
void vline_shade(image *pimage, int32 x, int32 y1, int32 y2, pix col1, pix col2)
{ int32 y;
  pix col;
  uint32 *buf;

  if(y1>y2){y = y1; y1 = y2; y2 = y; col = col1; col1 = col2; col2 = col;}

  /* pour le clipping avec l'image */
  if((y2<0) || (x<0) || (y1>=pimage->height) || (x>=pimage->width)) return;
  
  buf = (uint32 *)(&pimage->buf[(MAX(y1,0)*pimage->width)+x]);
  
  for(y = MAX(0,y1); y < MIN(pimage->height,y2+1); y++)
  {
    col.r = (col1.r * (y2-y) / (y2-y1)) + (col2.r * (y-y1) / (y2-y1));
    col.g = (col1.g * (y2-y) / (y2-y1)) + (col2.g * (y-y1) / (y2-y1));
    col.b = (col1.b * (y2-y) / (y2-y1)) + (col2.b * (y-y1) / (y2-y1));
    col.p = 255;
    *buf = *((uint32 *)&col);
    buf += pimage->width;
  }
}
/*****************************************************************/

/*******************************************************************/
/* Dessine un rectangle rempli entre les points (x1,y1) et (x2,y2) */
/* ce rectangle est rempli par un degrade lineaire de couleur      */
/*  definie comme sur le schema ci-dessous                         */
/*  col1 ---- col2                                                 */
/*   |          |                                                  */
/*  col3 ---- col4                                                 */
void rectangle_fill_shade(image *pimage, int32 x1, int32 y1, int32 x2, int32 y2,
  pix col1, pix col2, pix col3, pix col4)
{ int32 y,x;
  pix col_beg, col_end;
  
  if(x1>x2){x = x1; x1 = x2; x2 = x;}
  if(y1>y2){y = y1; y1 = y2; y2 = y;}
  
  if((x2<0) || (y2<0) || (x1>=pimage->width) || (y1>=pimage->height)) return;

  for(y = MAX(0,y1); y < MIN(pimage->height,y2+1); y++)
  {
    col_beg.r = (col1.r * (y2-y) / (y2-y1)) + (col3.r * (y-y1) / (y2-y1));
    col_beg.g = (col1.g * (y2-y) / (y2-y1)) + (col3.g * (y-y1) / (y2-y1));
    col_beg.b = (col1.b * (y2-y) / (y2-y1)) + (col3.b * (y-y1) / (y2-y1));

    col_end.r = (col2.r * (y2-y) / (y2-y1)) + (col4.r * (y-y1) / (y2-y1));
    col_end.g = (col2.g * (y2-y) / (y2-y1)) + (col4.g * (y-y1) / (y2-y1));
    col_end.b = (col2.b * (y2-y) / (y2-y1)) + (col4.b * (y-y1) / (y2-y1));
    
    hline_shade(pimage, x1, x2, y, col_beg, col_end);
  }
}
/*******************************/

/*******************************************************************************/
/* dessine une ligne avec antialiasime entre (x1,y1) et (x2,y2) de couleur col */
void line_aliased(image *pimage, int32 x1, int32 y1, int32 x2, int32 y2, pix col)
{ int32 x,y,deltax,deltay;
  int32 yval,xval;
  float trans;
  float x_real,y_real;

  deltay = (y2 > y1)? y2 - y1 : y1 - y2;
  deltax = (x2 > x1)? x2 - x1 : x1 - x2;

  /* si deltax > deltay, on avance en x */
  if(deltax > deltay)
  {
    /* tri pour que x1 <= x2 */
    if(x2 < x1)
    {
      x = x1; x1 = x2; x2 = x;
      y = y1; y1 = y2; y2 = y;
    }

    yval   = (y2 > y1)? 1 : -1;

    for(x=x1; x <= x2 - (deltax >> 1); x++)
    {
      y_real = (((float)((x2 - x)*y1 +  (x - x1)*y2))/((float)deltax));
      y = (int32)y_real;
      trans = ((float)y) - y_real;
      trans = ABS(trans);
      trans = 1.0 - trans;
      
      /* Dessine un pixel si dans l'image */
      if((x >= 0) && (x < pimage->width) && (y >= 0) && (y < pimage->height))
      {
	pimage->buf[y*pimage->width+x].r = 
          pimage->buf[y*pimage->width+x].r * (1.0 - trans) +
	  col.r * trans;
	pimage->buf[y*pimage->width+x].g = 
          pimage->buf[y*pimage->width+x].g * (1.0 - trans) +
	  col.g * trans;
	pimage->buf[y*pimage->width+x].b = 
          pimage->buf[y*pimage->width+x].b * (1.0 - trans) +
	  col.b * trans;
	pimage->buf[y*pimage->width+x].p = 255;
      }
      if((x >= 0) && (x < pimage->width) && (y+yval >= 0) && (y+yval < pimage->height))
      {
	pimage->buf[(y+yval)*pimage->width+x].r = 
          pimage->buf[(y+yval)*pimage->width+x].r * trans +
	  col.r * (1.0 - trans);
	pimage->buf[(y+yval)*pimage->width+x].g = 
          pimage->buf[(y+yval)*pimage->width+x].g * trans +
	  col.g * (1.0 - trans);
	pimage->buf[(y+yval)*pimage->width+x].b = 
          pimage->buf[(y+yval)*pimage->width+x].b * trans +
	  col.b * (1.0 - trans);
	pimage->buf[(y+yval)*pimage->width+x].p = 255;
      }

      if(x2-(x-x1) != x)
      {
      /* Dessine le pixel symetrique par rapport au milieu si il est dans l'image */
      if((x2-(x-x1) >= 0) && (x2-(x-x1) < pimage->width) && (y2-(y-y1) >= 0) && (y2-(y-y1) < pimage->height))
      {
	pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)].r = 
          pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)].r * (1.0 - trans) +
	  col.r * trans;
	pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)].g = 
          pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)].g * (1.0 - trans) +
	  col.g * trans;
	pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)].b = 
          pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)].b * (1.0 - trans) +
	  col.b * trans;
	pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)].p = 255;
      }
      if((x2-(x-x1) >= 0) && (x2-(x-x1) < pimage->width) && (y2-(y-y1)-yval >= 0) && (y2-(y-y1)-yval < pimage->height))
      {
	pimage->buf[(y2-(y-y1)-yval)*pimage->width+x2-(x-x1)].r = 
          pimage->buf[(y2-(y-y1)-yval)*pimage->width+x2-(x-x1)].r * trans +
	  col.r * (1.0 - trans);
	pimage->buf[(y2-(y-y1)-yval)*pimage->width+x2-(x-x1)].g = 
          pimage->buf[(y2-(y-y1)-yval)*pimage->width+x2-(x-x1)].g * trans +
	  col.g * (1.0 - trans);
	pimage->buf[(y2-(y-y1)-yval)*pimage->width+x2-(x-x1)].b = 
          pimage->buf[(y2-(y-y1)-yval)*pimage->width+x2-(x-x1)].b * trans +
	  col.b * (1.0 - trans);
	pimage->buf[(y2-(y-y1)-yval)*pimage->width+x2-(x-x1)].b = 255;
      }
      }

    }

  /* si deltay >= deltax, on avance de y */
  } else {

    /* tri pour que y1 <= y2 */
    if(y2 < y1)
    {
      x = x1; x1 = x2; x2 = x;
      y = y1; y1 = y2; y2 = y;
    }

    xval = (x2 > x1)? 1 : -1;

    for(y=y1; y <= y2 - (deltay >> 1); y++)
    {
      x_real = (((float)((y2 - y)*x1 +  (y - y1)*x2))/((float)deltay));
      x = (int32)x_real;
      trans = ((float)x) - x_real;
      trans = ABS(trans);
      trans = 1.0 - trans;

      /* Dessine le pixel si il est dans l'image */
      if((x >= 0) && (x < pimage->width) && (y >= 0) && (y < pimage->height))
      {
	pimage->buf[y*pimage->width+x].r = 
          pimage->buf[y*pimage->width+x].r * (1.0 - trans) +
	  col.r * trans;
	pimage->buf[y*pimage->width+x].g = 
          pimage->buf[y*pimage->width+x].g * (1.0 - trans) +
	  col.g * trans;
	pimage->buf[y*pimage->width+x].b = 
          pimage->buf[y*pimage->width+x].b * (1.0 - trans) +
	  col.b * trans;
	pimage->buf[y*pimage->width+x].b = 255;
      }
      if((x+xval >= 0) && (x+xval < pimage->width) && (y >= 0) && (y < pimage->height))
      {
	pimage->buf[y*pimage->width+x+xval].r = 
          pimage->buf[y*pimage->width+x+xval].r * trans +
	  col.r * (1.0 - trans);
	pimage->buf[y*pimage->width+x+xval].g = 
          pimage->buf[y*pimage->width+x+xval].g * trans +
	  col.g * (1.0 - trans);
	pimage->buf[y*pimage->width+x+xval].b = 
          pimage->buf[y*pimage->width+x+xval].b * trans +
	  col.b * (1.0 - trans);
	pimage->buf[y*pimage->width+x+xval].b = 255;
      }

      if(y2-(y-y1) != y)
      {
      /* Dessine le pixel symetrique par rapport au milieu si il est dans l'image */
      if((x2-(x-x1) >= 0) && (x2-(x-x1) < pimage->width) && (y2-(y-y1) >= 0) && (y2-(y-y1) < pimage->height))
      {
	pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)].r = 
          pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)].r * (1.0 - trans) +
	  col.r * trans;
	pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)].g = 
          pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)].g * (1.0 - trans) +
	  col.g * trans;
	pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)].b = 
          pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)].b * (1.0 - trans) +
	  col.b * trans;
	pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)].b = 255;
      }
      if((x2-(x-x1)-xval >= 0) && (x2-(x-x1)-xval < pimage->width) && (y2-(y-y1) >= 0) && (y2-(y-y1) < pimage->height))
      {
	pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)-xval].r = 
          pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)-xval].r * trans +
	  col.r * (1.0 - trans);
	pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)-xval].g = 
          pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)-xval].g * trans +
	  col.g * (1.0 - trans);
	pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)-xval].b = 
          pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)-xval].b * trans +
	  col.b * (1.0 - trans);
	pimage->buf[(y2-(y-y1))*pimage->width+x2-(x-x1)-xval].b = 255;
      }
      }
    }
  }

}
/*************************************************************/
