/****************************************************/
/* Quelques primitives de dessin                    */
/* image_draw_func.h                                */
/*                                                  */
/* Ecrit par : Daniel Lacroix (all rights reserved) */
/*                                                  */
/****************************************************/

#ifndef __IMAGE_DRAW_FUNC_H__
#define __IMAGE_DRAW_FUNC_H__

#include <image.h>

/* dessine un point aux coordonnees (x,y) de couleur col */
void put_pix(image *pimage, int32 x, int32 y, pix col);

/* renvoi le point aux coordonnees (x,y) */
pix get_pix(image *pimage, int32 x, int32 y);

/* dessine une ligne entre (x1,y1) et (x2,y2) de couleur col */
void line(image *pimage, int32 x1, int32 y1, int32 x2, int32 y2, pix col);

/* dessine une ligne avec antialiasime entre (x1,y1) et (x2,y2) de couleur col */
void line_aliased(image *pimage, int32 x1, int32 y1, int32 x2, int32 y2, pix col);

/* dessine une ligne horizontale entre (x1,y) et (x2,y) de couleur col */
void hline(image *pimage, int32 x1, int32 x2, int32 y, pix col);

/* dessine une ligne verticale entre (x,y1) et (x,y2) de couleur col */
void vline(image *pimage, int32 x, int32 y1, int32 y2, pix col);

/* dessine un rectangle vide entre (x1,y1) et (x2,y2) de couleur col */
void rectangle(image *pimage, int32 x1, int32 y1, int32 x2, int32 y2,pix col);

/* dessine un rectangle rempli */
void rectangle_fill(image *pimage, int32 x1, int32 y1, int32 x2, int32 y2,pix col);

/* repaint le fond avec col */
void clear_img(image *pimage,pix col);

/* dessine une ellipse de centre (cx,cy) et de rayon horizontal rx, vertical ry */
void ellipse(image *pimage, int32 cx, int32 cy, uint32 rx, uint32 ry, pix col);

/* Dessine une ellipse rempli de centre (cx,cy) et de */
/* rayon horizontal rx, vertical ry                   */
void ellipse_fill(image *pimage, int32 cx, int32 cy, uint32 rx, uint32 ry, pix col);

/* dessine une ligne horizontale entre (x1,y) de couleur col1    */
/* et (x2,y) de couleur col2 avec un degrade lineaire de couleur */
void hline_shade(image *pimage, int32 x1, int32 x2, int32 y, pix col1, pix col2);

/* dessine une ligne verticale entre (x,y1) de couleur col1      */
/* et (x,y2) de couleur col2 avec un degrade lineaire de couleur */
void vline_shade(image *pimage, int32 x, int32 y1, int32 y2, pix col1, pix col2);

/* Dessine un rectangle rempli entre les points (x1,y1) et (x2,y2) */
/* ce rectangle est rempli par un degrade lineaire de couleur      */
/*  definie comme sur le schema ci-dessous                         */
/*  col1 ---- col2                                                 */
/*   |          |                                                  */
/*  col3 ---- col4                                                 */
void rectangle_fill_shade(image *pimage, int32 x1, int32 y1, int32 x2, int32 y2,
  pix col1, pix col2, pix col3, pix col4);

#endif /* __IMAGE_DRAW_FUNC_H__ */
