#include <image_trans.h>
#include <image_draw_func.h>
#include <math.h>

/***************************************************/
/* Redimentionne l'image psrc et place le resultat */
/* a la postion (0,0) dans pdest                   */
void image_resize(image *pdest, image *psrc, int32 width, int32 height)
{ int32 vx,vy;
  pix   vcol,vcol1,vcol2,vcol3,vcol4;
	float rx,ry;
	float width_scale, height_scale;
	float x_dist, y_dist;

  width_scale  = (float)psrc->width  / (float)width;
  height_scale = (float)psrc->height / (float)height;

	for(vy = 0;vy < height; vy++)  
	{
  	for(vx = 0;vx < width; vx++)
		{
		  rx = vx * width_scale;
		  ry = vy * height_scale;
		  vcol1 = get_pix(psrc, (int32)rx, (int32)ry);
		  vcol2 = get_pix(psrc, ((int32)rx)+1, (int32)ry);

		  vcol3 = get_pix(psrc, (int32)rx, ((int32)ry)+1);
		  vcol4 = get_pix(psrc, ((int32)rx)+1, ((int32)ry)+1);

      x_dist = rx - ((float)((int32)rx));
      y_dist = ry - ((float)((int32)ry));
      vcol.r = (vcol1.r*(1.0-x_dist) + vcol2.r*(x_dist))*(1.0-y_dist) + 
			         (vcol3.r*(1.0-x_dist) + vcol4.r*(x_dist))*(y_dist);
      vcol.g = (vcol1.g*(1.0-x_dist) + vcol2.g*(x_dist))*(1.0-y_dist) + 
			         (vcol3.g*(1.0-x_dist) + vcol4.g*(x_dist))*(y_dist);
      vcol.b = (vcol1.b*(1.0-x_dist) + vcol2.b*(x_dist))*(1.0-y_dist) + 
			         (vcol3.b*(1.0-x_dist) + vcol4.b*(x_dist))*(y_dist);

/*      vcol.r = ((float)vcol1.r)*(1.0-x_dist) + ((float)vcol2.r)*x_dist;
      vcol.g = ((float)vcol1.g)*(1.0-x_dist) + ((float)vcol2.g)*x_dist;
      vcol.b = ((float)vcol1.b)*(1.0-x_dist) + ((float)vcol2.b)*x_dist;
*/
		  put_pix(pdest, vx, vy, vcol);
		}
	}
}
/***************************************************/

/***************************************************/
/* Redimentionne l'image psrc et place le resultat */
/* a la postion (0,0) dans pdest                   */
image_resize_dct(image *pdest, image *psrc, int32 width, int32 height)
{
  float dct_coeff[8][8];  /* 8 fonctions de bases definie par 8 valeurs */
	uint8 val[8][8];
  float dct_res1[8][8];   /* coefficient 2d */
  float dct_res[8][8];    /* coefficient 3d qui correspondent a nos pixels */
	
	/* initialise le tableau des fonctions de bases */
	double f1,f2,tmp;
  int x,y,coeff,i;

  int32 vx,vy;
	float rx,ry;
	float width_scale, height_scale;
	float x_dist, y_dist;
	pix   vcol;

  void init_dct(void)
	{
  	f1 = 2.0*atan(1.0)/8.0;
  	for(y=0;y<8;y++){         /* de la fonction 0 a la 7              */
    	f2=(double)y*f1;
    	for(x=0;x<8;x++){       /* du point 0 au point 7 de la fonction */
      	tmp=cos((double)(2*x+1)*f2);
      	if(y==0) dct_coeff[y][x]=(float)(1.0/sqrt(2.0));
      	else     dct_coeff[y][x]=(float)tmp;
    	}
  	}
  }

  void calcul_dct(void)
	{
  	/* ON PREND EN ENTREE pix le tableau de pix[y][x] et on retourn dct_res
	  	 les coeffs */
  	/* fait la DCT par lignes */
  	for(y = 0; y < 8; y++)
		{
  		for(coeff = 0; coeff < 8; coeff++)
			{
    		dct_res1[y][coeff] = 0.0;

    		for(x = 0; x < 8; x++)
				{
      		dct_res1[y][coeff] +=
        		((float)val[y][x])*(float)dct_coeff[coeff][x];
    		}    /* integrale de (Pix*dct_coeff) pour chaque fonction de base */
  		}
		}
  	/* fait la DCT par colonnes */
  	for(x = 0; x < 8; x++)
		{
  		for(coeff = 0; coeff < 8; coeff++)
			{
    		dct_res[coeff][x] = 0.0;

    		for(i = 0; i < 8; i++)
				{
      		dct_res[coeff][x] +=
        		((float)val[i][x])*(float)dct_coeff[coeff][i];
    		}
  		}
		}
  } /* fin de calcul_dct() */

  uint8 calcul_pix(float px, float py)
	{ uint8 res;
	
    float temp;
		float dct_coeff_hor[8],dct_coeff_ver[8];

    f1 = 2.0*atan(1.0)/8.0;
		for(y=0;y<8;y++)
		{
		  f2=(double)y*f1;
      tmp=cos((double)(2.0*(4.0+px)+1.0)*f2);
      if(y==0) dct_coeff_hor[y]=(float)(1.0/sqrt(2.0));
      else     dct_coeff_hor[y]=(float)tmp;
      tmp=cos((double)(2.0*(4.0+py)+1.0)*f2);
      if(y==0) dct_coeff_ver[y]=(float)(1.0/sqrt(2.0));
      else     dct_coeff_ver[y]=(float)tmp;
    }
		
		temp = 0.0;
		for(y=0;y<8;y++)
		  for(x=0;x<8;x++)
		    temp += dct_coeff_hor[x] * dct_coeff_ver[y] * dct_res[x][y];

    if(temp > 255.0) res = 255;
		else if(temp < 0.0) res = 0;
		else res = (uint8)temp;
	  return(res);
	}
	
	/* corp de la procedure */
	init_dct();
	
	width_scale  = (float)psrc->width  / (float)width;
  height_scale = (float)psrc->height / (float)height;
	
	for(vy = 0; vy < height; vy++)  
	{
  	for(vx = 0; vx < width; vx++)
		{
		  rx = vx * width_scale;
			ry = vy * height_scale;

      x_dist = rx - ((float)((int32)rx));
      y_dist = ry - ((float)((int32)ry));
      /* pour le rouge */
			for(y=0;y<8;y++)
			  for(x=0;x<8;x++)
			    val[y][x] = get_pix(psrc, ((int32)rx)-x+4, ((int32)ry)-y+4).r;
      calcul_dct();
			vcol.r = calcul_pix(x_dist, y_dist);
      /* pour le vert */
			for(y=0;y<8;y++)
			  for(x=0;x<8;x++)
			    val[y][x] = get_pix(psrc, ((int32)rx)-x+4, ((int32)ry)-y+4).g;
      calcul_dct();
			vcol.g = calcul_pix(x_dist, y_dist);
      /* pour le bleu */
			for(y=0;y<8;y++)
			  for(x=0;x<8;x++)
			    val[y][x] = get_pix(psrc, ((int32)rx)-x+4, ((int32)ry)-y+4).b;
      calcul_dct();
			vcol.b = calcul_pix(x_dist, y_dist);
			/* on place le pixel calcule */
		  put_pix(pdest, vx, vy, vcol);
		}
	}

}
/***************************************************/

