#include <stdio.h>
#include <general.h>
#include <image.h>
#include <image_trans.h>
#include <image_from_jpeg.h>

enum { NEAREST, BILINEAR, DCT };

void usage(void)
{
  printf(
  "Usage: resize_image [OPTION] SRC_FILE NEW_WIDTH NEW_HEIGHT -o DST_FILE\n"\
  "resize the SRC_FILE jpeg image into DST_FILE with the\n"\
  "dimension (NEW_WIDTHxNEW_HEIGHT)\n\n"
  "  -b, --bilinear  resize using a bilinear interpolation (DEFAULT)\n"\
  "  -n, --nearest   resize using the nearest dot\n"\
  "  -d, --dct       resize using a 2D DCT interpolation (very long)\n"\
  "      --help      display this help and exit\n"
  );
}

int main(int argc, char **argv)
{
  image *vimage, *vimage_resize;
  int i;
  char *file_src = NULL;
  char *file_dst = NULL;
  uint8 mode = BILINEAR;
  int32 width  = 0;
  int32 height = 0;

  /* read command line arguments */  
  for(i = 1; i < argc; i++)
  {
    if(!strcmp(argv[i],"--help")) { usage(); exit(2); }
    else if((!strcmp(argv[i],"-b")) || (!strcmp(argv[i],"--bilinear"))) mode = BILINEAR;
    else if((!strcmp(argv[i],"-d")) || (!strcmp(argv[i],"--dct"))) mode = DCT;
    else if((!strcmp(argv[i],"-n")) || (!strcmp(argv[i],"--nearest"))) mode = NEAREST;
    else if(!strcmp(argv[i],"-o")) {
      i++;
      if((i < argc) && (file_dst == NULL)) {
        file_dst = argv[i];
      } else { usage(); exit(2); }
    } else {
      if(file_src == NULL) {
        file_src = argv[i];
        /* file src MUST be followed by its new dimension */
        i++; if(i >= argc) { usage(); exit(2); }
        width  = atoi(argv[i]);
        i++; if(i >= argc) { usage(); exit(2); }
        height = atoi(argv[i]);
        /* negative or null is not allowed */
        if((width <= 0) || (height <= 0)) { usage(); exit(2); }
      } else { usage(); exit(2); }
    }
  }

  if((width <= 0) || (height <= 0) || (file_src == NULL) || (file_dst == NULL))
  { usage(); exit(2); }


  /* load the JPEG src image */
  vimage = image_new_from_jpeg(file_src);
  if(vimage == NULL)
  { printf("ERROR while loadind source file \"%s\"\n",file_src); exit(2); }

  /* create the destination image */  
  vimage_resize = image_new(width,height);

  switch(mode)
  {
    case NEAREST  :
      image_resize_nearest(vimage_resize,vimage,width,height);
      break;
    case BILINEAR :
      image_resize_bilinear(vimage_resize,vimage,width,height);
      break;
    case DCT      :
      printf("Be patient it can be long (1024x768 destination image takes"\
             " 15 minutes on a PII 333Mhz)\n\n");
      image_resize_dct(vimage_resize,vimage,width,height);
      break;
  }

  /* save the resulting image */  
  image_save_to_jpeg(vimage_resize, file_dst);

  return(0);
}
