/****************************************************/
/* Gestion de listes doublement chainees            */
/* list.h                                           */
/*                                                  */
/* Ecrit par : Daniel Lacroix (all rights reserved) */
/*                                                  */
/****************************************************/

#ifndef __LIST_H__
#define __LIST_H__

#include <general.h>

/* pour les listes chainees */
typedef struct _List List;

struct _List {
  pointer data;
  List *next;
  List *prev;
};

/* Rajoute un element en tete                                        */
/* Si votre liste n'a pas d'element factice en tete, vous devez      */
/* recuperer le nouveau point d'entree de la liste qui est retourne. */
List *list_prepend(List *pList, pointer data);

/* Rajoute un element en queue                                       */
/* Si votre liste n'a pas d'element factice en tete, vous devez      */
/* recuperer le nouveau point d'entree de la liste qui est retourne. */
List *list_append(List *pList, pointer data);

/* Cree une nouvelle liste avec un element factice en tete.       */
/* Pour cree une liste sans element factice il suffit d'utiliser  */
/* un pointeur NULL. L'element factice permet d'avoir une adresse */
/* de debut de la liste qui ne change pas quand on insere des     */
/* elements dans la liste.                                        */
List *list_new(void);

/* Libere la liste (pas les elements) */
void list_free(List *pList);

/* Libere la liste et appele la procedure free_func pour chaque elements */
void list_free_full(List *pList, void (* free_func)(pointer data));

/* Libere la liste et appele la procedure free (du C) pour chaque elements */
void list_free_full_simple(List *pList);

/* Renvoi le nombre d'element contenue dans la liste   */
/* Si la liste a un fictif en tete alors il est compte */
uint32 list_nb_item(List *pList);

/* Libere un element de la liste (pas ce que l'element pointe si c'est un pointeur) */
List *list_free_item(List *pList, pointer data);

/* Libere un element de la liste (pas ce que l'element pointe si c'est un pointeur) */
/* Si l'element a ete libere alors done == TRUE sinon FALSE                         */
List *list_free_item_with_check(List *pList, pointer data, boolean *done);

/* Renvoi l'element a la position item_nb dans la liste pList */
/* Si la liste a un fictif en tete alors il est compte        */
/* de meme que si l'element item_nb n'exite pas, la fonction  */
/* renvoi NULL. item_nb commence a 1.                         */
pointer list_item(List *pList, uint32 item_nb);

/* Renvoi l'element a la derniere position de la liste pList.         */
/* Renvoi NULL si la liste est vide ou le fictif si il n'y a que lui. */
List *list_last(List *pList);

/* Rajoute un element apres l'element before. Si l'element before    */
/* n'est pas trouve alors le nouvel element sera rajoute a la fin    */
/* Si votre liste n'a pas d'element factice en tete, vous devez      */
/* recuperer le nouveau point d'entree de la liste qui est retourne. */
List *list_insert_after_item(List *pList, pointer before, pointer data);

#endif /* __LIST_H__ */
